package App::Mangemanche;
use strict;
use warnings;
use Getopt::Long qw(:config gnu_getopt no_ignore_case require_order);
use Pod::Man;
use Pod::Usage;
use Pod::Find qw(pod_where);
use Net::Ping903;
use App::Mangemanche::Command ':exit_codes';
use File::Basename;

our $VERSION = '0.4.90';

my $DEFAULT_URL = 'http://localhost:8080';

sub new {
    my $class = shift;
    my $config_file;
    my $url;
    my $config = {};

    GetOptions(
        'shorthelp|?' => sub {
            pod2usage(-input => pod_where({-inc => 1}, $class),
                      -verbose => 99,
                      -sections => [qw(NAME SYNOPSIS COMMANDS)],
                      -exitstatus => 'NOEXIT');
	    $class->list_commands;
	    exit(EX_OK);
        },
        'help' => sub {
	    App::Mangemanche::Command::mandoc($class, basename($0)),
        },
        'usage' => sub {
            pod2usage(-exitstatus => EX_OK,
                      -input => pod_where({-inc => 1}, $class),
                      -verbose => 0)
	},
        'config|c=s' => \$config_file,
	'url|u=s' => \$url	
    ) or exit(EX_USAGE);

    if ($config_file) {
	if (-f $config_file) {
	    $config = $class->readconfig($config_file);
	} else {
	    die "configuration file $config_file does not exists\n";
	}
    } else {
	$config_file = '/etc/ping903.conf';
	if (-f $config_file) {
	    $config = $class->readconfig($config_file);
	}
    }

    if ($url) {
	$config->{baseurl} = $url;
    } elsif (!$config->{baseurl}) {
	$config->{baseurl} = $DEFAULT_URL;
    }

    unless ($config->{baseurl} =~ m{^https?://}) {
	$config->{baseurl} = "http://$url";
    }

    my $agent = new Net::Ping903($config->{baseurl});
    
    my $com = shift @ARGV;
    die "no command name\n" unless $com;

    my $modname = __PACKAGE__ . '::Command::' . $com;
    my $modpath = $modname;
    $modpath =~ s{::}{/}g;
    $modpath .= '.pm';
    my $cmd;
    eval {
	require $modpath;
        $cmd = $modname->new($com, $agent);
    };
    if ($@) {
	if ($@ =~ /Can't locate $modpath/) {
	    die "unknown command: $com\n"
	}
	die $@;
    }
    return $cmd;
}

sub readconfig {
    my ($class, $file) = @_;
    my $config = {};
    if (open(my $fh, '<', $file)) {
	while (<$fh>) {
	    chomp;
	    s/^\s+//;
	    s/\s+$//;
	    next if /^(#.*)?$/;
	    if (m{^listen\s+(.+)$}) {
		$config->{baseurl} = $1;
		last;
	    }
	}
	close $fh;
    } else {
	die "$file: file doesn't exist\n";
    }
    return $config;
}

sub list_commands {
    my $class = shift;
    my @classpath = (split(/::/, $class), 'Command');

    print "\nAvailable commands are:\n";
    foreach my $mod (sort
		     map {
	                 my $name = basename($_);
			 if (exists($INC{File::Spec->catfile(@classpath,$name)})) {
			     ()
			 } else {
			     eval {
				     require $_;
			     };
			     $name =~ s/\.pm$//;			     
			     $@ ? () : [$name, $_];
			}
		     }
	             map {
		           glob File::Spec->catfile($_, @classpath, '*.pm')
                     } @INC) {

	my $s;
	open(my $fh, '>', \$s);
        pod2usage(-input => $mod->[1],
		  -output => $fh,
                      -verbose => 99,
                      -sections => ['NAME'],
                      -exitstatus => 'NOEXIT');
	close $fh;
	my (undef, $descr) = split("\n", $s||'');
	unless ($descr) {
	    $descr = '    ' . $mod->[0]
	} 
	print "$descr\n";
    }
}

1;

=head1 NAME

mangemanche - interface to B<ping903> daemon.
    
=head1 SYNOPSIS

B<mangemanche>
[B<-c> I<FILE>]
[B<-u> I<FILE>]
[B<--config=>I<FILE>]
[B<--url=>I<FILE>]    
I<COMMAND>
I<ARG>...

B<mangemanche>
[B<-?>]
[B<--help>]
[B<--usage>]    
    
=head1 DESCRIPTION

=head1 OPTIONS

    
